@@ -5,14 +5,14 @@ Official documentation for Bigbuy API endpoints can be found at: https://api.big
 """
 import base64
 import mimetypes
-import warnings
-from typing import Optional, Dict, Any, Union, Iterable, List, cast
+from http.cookiejar import DefaultCookiePolicy
+from typing import Optional, Dict, Union, Iterable, List, cast
 
 import requests
-from api_session import APISession
+from api_session import APISession, JSONDict
 
 from . import __version__
-from .exceptions import raise_for_response
+from .exceptions import raise_for_response, BBError
 from .rate_limit import RateLimit
 
 __all__ = ['BigBuy']
@@ -54,6 +54,8 @@ class BigBuy(APISession):
         self.retry_on_rate_limit = retry_on_rate_limit
         self.max_retry_on_rate_limit = max_retry_on_rate_limit
         self.headers.setdefault('Authorization', f'Bearer {app_key}')
+        # Reject all cookies by default. They are not necessary for the API usage (and not documented).
+        self.cookies.set_policy(DefaultCookiePolicy(allowed_domains=[]))
 
     def __repr__(self):
         attrs = f" key={self.app_key[:10]}…" if self.app_key else ""
@@ -268,111 +270,139 @@ class BigBuy(APISession):
         """Get a single variation."""
         return self.get_json_api('shipping/carriers', params=params)
 
-    def get_shipping_order(self, order: Dict[str, Any]):
+    def get_shipping_order(self, order: JSONDict):
         """Get the list of available shipping options with the calculated weight and cost in Kg and € respectively,
         for the given order.
 
         Example order:
             {"delivery":{"isoCountry":"ES","postcode":"46005"},"products":[{"reference":"V1300179","quantity":1}]}
         """
-        if "order" in order:  # pragma: nocover
-            warnings.warn("Calling get_shipping_order({\"order\": order}) is deprecated;"
-                          " use get_shipping_order(order) instead.",
-                          DeprecationWarning)
-            order_payload = order
-        else:
-            order_payload = {"order": order}
-        return self.post_api('shipping/orders', json=order_payload).json()
+        return self.post_api('shipping/orders', json={"order": order}).json()
 
     # order
-    def check_order(self, order: Dict[str, Any], **params):
-        """Check/simulate an order and return the total order to pay.
+    def check_order(self, order: JSONDict, **params) -> dict:
+        """Check/simulate an order and return the total amount to pay.
 
         Example order:
 
-        {
-          "internalReference": "123456",
-          "language": "es",
-          "paymentMethod": "moneybox",
-          "carriers": [
             {
-              "name": "correos"
-            },
-            {
-              "name": "chrono"
+              "internalReference": "123456",
+              "language": "es",
+              "paymentMethod": "moneybox",
+              "carriers": [
+                { "name": "correos" },
+                { "name": "chrono" }
+              ],
+              "shippingAddress": {
+                "firstName": "John",
+                "lastName": "Doe",
+                "country": "ES",
+                "postcode": "46005",
+                "town": "Valencia",
+                "address": "C/ Altea",
+                "phone": "664869570",
+                "email": "john@email.com",
+                "comment": ""
+              },
+              "products": [
+                {
+                  "reference": "F1505138",
+                  "quantity": 4
+                }
+              ]
             }
-          ],
-          "shippingAddress": {
-            "firstName": "John",
-            "lastName": "Doe",
-            "country": "ES",
-            "postcode": "46005",
-            "town": "Valencia",
-            "address": "C/ Altea",
-            "phone": "664869570",
-            "email": "john@email.com",
-            "comment": ""
-          },
-          "products": [
-            {
-              "reference": "F1505138",
-              "quantity": 4
-            }
-          ]
-        }
-        """
-        if "order" in order:  # pragma: nocover
-            warnings.warn("Calling check_order({\"order\": order}) is deprecated; use check_order(order) instead.",
-                          DeprecationWarning)
-            order_payload = order
-        else:
-            order_payload = {"order": order}
-        return self.post_api('order/check', json=order_payload, **params).json()
 
-    def create_order(self, order: Dict[str, Any], **params):
+        Example response:
+
+            {
+              "totalWithoutTaxesAndWithoutShippingCost": 4.52,
+              "totalWithoutTaxes": 8.52,
+              "total": 9.809999999999999
+            }
         """
-        Submit an order.
+        return self.post_api('order/check', json={"order": order}, **params).json()
+
+    def check_multi_shipping_order(self, order: JSONDict, **params) -> Dict[str, List[JSONDict]]:
+        """
+        Check/simulate an order and return the total to pay. This is the multi-shipping version, which is required for
+        some references.
+
+        See `check_order` for the input format. The response differs because it splits the order in multiple sub-orders,
+        each one with its check result.
+        Example response:
+            {
+              "orders": [
+                {
+                  "productReferences": [ "S4602570" ],
+                  "totalWithoutTaxesAndWithoutShippingCost": 4.52,
+                  "totalWithoutTaxes": 8.52,
+                  "total": 9.809999999999999,
+                  "warehouse": 1
+                },
+                {
+                  "productReferences": [ "S7106391" ],
+                  "totalWithoutTaxesAndWithoutShippingCost": 109.2,
+                  "totalWithoutTaxes": 109.2,
+                  "total": 132.13,
+                  "warehouse": 3
+                }
+              ],
+              "errors": []
+            }
+        """
+        return self.post_api('order/check/multishipping', json={"order": order}, **params).json()
+
+    def create_order(self, order: JSONDict, **params):
+        """
+        Submit an order and return the raw response.
 
         Example order:
-        order = {
-          "internalReference": "123456",
-          "language": "es",
-          "paymentMethod": "moneybox",
-          "carriers": [
-            {
-              "name": "correos"
-            },
-            {
-              "name": "chrono"
+            order = {
+              "internalReference": "123456",
+              "language": "es",
+              "paymentMethod": "moneybox",
+              "carriers": [
+                {
+                  "name": "correos"
+                },
+                {
+                  "name": "chrono"
+                }
+              ],
+              "shippingAddress": {
+                "firstName": "John",
+                "lastName": "Doe",
+                "country": "ES",
+                "postcode": "46005",
+                "town": "Valencia",
+                "address": "C/ Altea",
+                "phone": "664869570",
+                "email": "john@email.com",
+                "comment": ""
+              },
+              "products": [
+                {
+                  "reference": "F1505138",
+                  "quantity": 4
+                }
+              ]
             }
-          ],
-          "shippingAddress": {
-            "firstName": "John",
-            "lastName": "Doe",
-            "country": "ES",
-            "postcode": "46005",
-            "town": "Valencia",
-            "address": "C/ Altea",
-            "phone": "664869570",
-            "email": "john@email.com",
-            "comment": ""
-          },
-          "products": [
-            {
-              "reference": "F1505138",
-              "quantity": 4
-            }
-          ]
-        }
         """
-        if "order" in order:  # pragma: nocover
-            warnings.warn("Calling create_order({\"order\": order}) is deprecated; use create_order(order) instead.",
-                          DeprecationWarning)
-            order_payload = order
-        else:
-            order_payload = {"order": order}
-        # NOTE(BF): we must return the raw response because we need the headers to parse 'Location'
-        return self.post_api('order/create', json=order_payload, **params)
+        # NOTE: we must return the raw response because we need the headers to parse 'Location'
+        return self.post_api('order/create', json={"order": order}, **params)
+
+    def create_multi_shipping_order(self, order: JSONDict, **params) -> Dict[str, List[JSONDict]]:
+        """
+        Submit an order. This is the multi-shipping version, which is required for some references.
+
+        See `create_order` for the input format.
+
+        Example response:
+            {"orders":[{"productReferences":["S1"],"id":"123","warehouse":1,"url":"\\/rest\\/order\\/123"},
+                       {"productReferences":["S2", "S3"],"id":"124","warehouse":3,"url":"\\/rest\\/order\\/124"}],
+             "errors":[]}
+        """
+        return self.post_api('order/create/multishipping', json={"order": order}, **params).json()
 
     def create_order_id(self, order: dict, **params) -> str:
         """Like create_order(), but return the order id."""
@@ -386,11 +416,24 @@ class BigBuy(APISession):
         #     'Set-Cookie': 'secure_key=16...065; expires=Thu, 16-Apr-2020 07:24:56 GMT; Max-Age=604800; path=/',
         # }
         # the id of the bigbuy order is only known in the location url in the headers
-        order_id = response.headers["Location"].replace("/rest/order/", "")
-        return order_id
+        return _get_order_id_from_response_redirect(response)
+
+    def create_multi_shipping_order_ids(self, order: dict, **params) -> List[str]:
+        """
+        Like `create_multi_shipping_order()`, but return the order ids.
+        This checks if the `errors` array is not empty, and raises a `BBError` if so.
+        """
+        creation_response = self.create_multi_shipping_order(order, **params)
+        if creation_response["errors"]:
+            raise BBError("Multi-shipping order errors: %s" % creation_response)
+
+        return [order["id"] for order in creation_response["orders"]]
 
     def get_order_by_customer_reference(self, reference: str, **params):
-        """Get order information by customer reference."""
+        """
+        Get order information by customer reference. Note that this doesn’t support multi-shipping orders and returns
+        only one of the orders matching the customer reference.
+        """
         return self.get_json_api(f'order/reference/{reference}', **params)
 
     def get_order_by_id(self, order_id: Id, **params):
@@ -548,3 +591,7 @@ class BigBuy(APISession):
         The format is the same as ``get_product_taxonomies``
         """
         return self.get_json_api(f"catalog/productstaxonomies", **params)
+
+
+def _get_order_id_from_response_redirect(response: requests.Response):
+    return response.headers["Location"].replace("/rest/order/", "")
